package domain

// ===================錢包金額種類===================

type WalletType int64

// 錢包種類
const (
	// WalletAvailableType 錢包可動用的金額
	WalletAvailableType WalletType = iota + 1

	// WalletFreezeType 交易過程，錢包被凍結的金額
	WalletFreezeType

	// WalletUnconfirmedType 已提交的交易但還未確認完成交易的金額
	WalletUnconfirmedType

	// WalletContractAvailableType 合約/交易可用餘額
	WalletContractAvailableType

	// WalletContractFreezeType 合約/交易凍結餘額
	WalletContractFreezeType
)

var walletTypeToName = map[WalletType]string{
	WalletAvailableType:         "available",
	WalletFreezeType:            "freeze",
	WalletUnconfirmedType:       "unconfirmed",
	WalletContractAvailableType: "contract_available",
	WalletContractFreezeType:    "contract_freeze",
}

var nameToWalletType = map[string]WalletType{
	"available":          WalletAvailableType,
	"freeze":             WalletFreezeType,
	"unconfirmed":        WalletUnconfirmedType,
	"contract_available": WalletContractAvailableType,
	"contract_freeze":    WalletContractFreezeType,
}

// Name returns the name associated with the WalletType.
func (w WalletType) Name() string {
	return walletTypeToName[w]
}

// GetWalletTypeByName returns the WalletType associated with the given name.
func GetWalletTypeByName(name string) WalletType {
	return nameToWalletType[name]
}

var AllWalletType = []WalletType{
	WalletAvailableType, WalletFreezeType, WalletUnconfirmedType,
	WalletContractAvailableType, WalletContractFreezeType,
}
